#include "mpicomm.hpp"
#include <slepceps.h>
#include <armadillo>
#include <iostream>
#include "tricount.hpp"

template <class INT, class FLOAT>
void armatocoo(arma::sp_fmat &A, INT *row_idx, INT *col_idx, FLOAT *val, bool adjacency = 0, INT *nnz_per_row = NULL) {
	arma::sp_fmat::iterator start_it = A.begin();
	arma::sp_fmat::iterator end_it = A.end();
	int j = 0;
	for (int i = 0; i < A.n_rows; i++) {
		nnz_per_row[i] = 0;
	}
	for (arma::sp_fmat::iterator it = start_it; it != end_it; ++it) {
		row_idx[j] = it.row();
		col_idx[j] = it.col();
		(adjacency) ? val[j++] = 1 : val[j++] = (*it);
		nnz_per_row[it.row()]++;
	}
}

int main(int argc, char *argv[]) {
	char *help = "Triangle counting using eigen values";
	MPICommunicator mpicomm(argc, argv);
	int globalm = 16000;
	int globaln = 16000;
	double sparsity = 0.01;
	int localm = globalm / mpicomm.size();
	int localn = globaln;
	//construct uniform random matrix
	arma::sp_fmat Arows = arma::sprandu<arma::sp_fmat>(localm, localn, sparsity);
	int nnzs =  Arows.n_nonzero;
	int rows = Arows.n_rows;
	int cols = Arows.n_cols;
	std::cout << "Arows::" << Arows.n_rows << "x" << Arows.n_cols << std::endl;
	std::cout << "nnzs::" << nnzs << "::rows::" << rows << "::cols::"  << cols << std::endl;
	//change it to coo matrix
	PetscScalar* val;
	PetscInt* col_idx;
	PetscInt* row_idx;
	PetscInt* nnz_per_row = NULL;

	PetscMalloc1(nnzs, &val);
	PetscMalloc1(nnzs, &col_idx);
	PetscMalloc1(nnzs, &row_idx);
	PetscMalloc1(Arows.n_rows, &nnz_per_row);
	armatocoo<PetscInt, PetscScalar>(Arows, row_idx, col_idx, val, 1, nnz_per_row);
	//construct triangle count
	SlepcInitialize(&argc, &argv, NULL, help);
	TriangleCount tricount(rows, cols, nnzs,
	                       row_idx, col_idx, val, mpicomm);
	tricount.count();
	if (mpicomm.rank() == 0) {
		std::cout << "total triangles::" << tricount.num_triangles() << std::endl;		
	}

}